/* this is objects.c file
 * this is part of the libspopc library sources
 * copyright  2002 Benoit Rouits <brouits@free.fr>
 * released under the terms of GNU LGPL
 * (GNU Lesser General Public Licence).
 * libspopc offers simple API for a pop3 client (MTA).
 * See RFC 1725 for pop3 specifications.
 * more information on http://brouits.free.fr/libspopc/
 */

/***************************************************
 * high-level objects and methods for a SIMPLE MTA *
 ***************************************************/

typedef enum{AUTHORIZATION,TRANSACTION,UPDATE}popstate;
/* pop server states definition from RFC 1725*/

typedef struct{
	int sock;/* socket descriptor */
	struct sockaddr_in* connection;
	struct hostent* server;
	popstate state;/* pop server state */
	int* list;/* pop messages size list */
	char** uidl;/* pop messages signature list */
	int bytes;/* total stored (in bytes) on pop server */
	int last;/* last message id */
	int cur;/* last retrieved message id */
	int del;/* 0|1 flag to ask deletion of retrieved messages */
}popsession;

char* popbegin(char* servername,char* user,char* pass, popsession* s){
/* prepares, connect and get lists of messages stored on pop server */
/* you must give a valid servername, user and pass */
/* returns an error message if a problem occurs */
int r=0;
char* resp=NULL;
char* err=NULL;
	
	if(!(servername && user && pass)){
		err=strdup("NULL args !");
		return(err);
	}
	s=(popsession*)malloc(sizeof(popsession));
	if(!s){
		err=strdup("malloc");
		return(err);
	}
	s->connection=(struct sockaddr_in*)malloc(sizeof(struct sockaddr_in));
	if(!(s->connection)){
		err=strdup("malloc");
		free(s);s=NULL;
		return(err);
	}
	r=pop3_prepare(servername,110,s->connection,s->server);
	if(r==-1){
		err=strdup("pop3_prepare");
		free(s->connection);
		free(s);s=NULL;
		return(err);
	}
	s->sock=r;
	resp=pop3_connect(s->sock,s->connection);
	if(!resp){
		err=strdup("pop3_connect");
		pop3_disconnect(s->sock);
		free(s->connection);
		free(s);s=NULL;
		return(err);
	}
	free(resp);
	resp=pop3_user(s->sock,user);
	if((!resp) || pop3_error(resp)){
		pop3_disconnect(s->sock);
		free(s->connection);
		free(s);s=NULL;
		err=resp?resp:strdup("pop3_user");
		return(err);
	}
	free(resp);
	resp=pop3_pass(s->sock,pass);
	if((!resp) || pop3_error(resp)){
		pop3_disconnect(s->sock);
		free(s->connection);
		free(s);s=NULL;
		err=resp?:resp:strdup("pop3_pass");
		return(err);
	}
	s->state=TRANSACTION;
	free(resp);
	resp=pop3_stat(s->sock);
	if((!resp) || pop3_error(resp)){
		pop3_disconnect(s->sock);
		free(s->connection);
		free(s);s=NULL;
		err=resp?resp:strdup("pop3_stat");
		return(err);
	}
	s->bytes=stat2bytes(resp);
	s->last=stat2last(resp);
	free(resp);
	resp=pop3_list(s->sock,0);
	if((!resp) || pop3_error(resp)){
		pop3_disconnect(s->sock);
		free(s->connection);
		free(s);s=NULL;
		err=resp?resp:strdup("pop3_list");
		return(err);
	}
	s->list=list2array(resp);
	free(resp);
	resp=pop3_uidl(s->sock,0);
	if((!resp) || pop3_error(resp)){
		pop3_disconnect(s->sock);
		free(s->connection);
		free(s);s=NULL;
		err=resp?resp:strdup("pop3_uidl");
		return(err);
	}
	s->uidl=uidl2array(resp);
	s->cur=0;
	free(resp);
	return(NULL);
}

char* popgetmsg(popsession* session){
/* returns a new message (from 1 to last message) or NULL if no more message */
char* resp;
char* msg;

	if(!session){
		return(NULL);
	}
	if(session->cur >= session->last){
		return(NULL);
	}
	session->cur++;
	resp=pop3_retr(session->sock,session->cur);
	if((!resp) || pop3_error(resp)){
		return(NULL);/* any suggestions ? */
	}
	msg=retr2msg(resp);
	if(!msg){
		msg=resp;
	}else{
		free(resp);
	}
	if(session->del){
		pop3_dele(session->sock,session->cur);
	}
	return(msg);	
}

void popend(popsession* session){
/* quit and destroys pop session */
int i;

	if(!session){return;}
	pop3_quit(session->sock);
	pop3_disconnect(session->sock);
	free(session->connection);
	free(session->server);
	free(session->list);
	for(i=0;i<=session->last;i++){
		free(session->uidl[i]);
	}
	free(session->uidl);
	free(session);
	return;
}

#define popgetbytes(s) ((s)->bytes)
/* gives the total stored data size (in bytes) on the pop server */
/* arg is type 'popsession*'; return is type 'int' */

#define popgetnum(s) ((s)->last)
/* returns the number of messages stored on the pop server before any operation */
/* arg is type 'popsession*'; return is type 'int' */

#define popsetdel(s) ((s)->del=1)
/* asks the session to delete any retrieved messages on the server */
/* arg is type 'popsession*' */

#define popmsgsize(s,i) ((s)->list[(i)])
/* gives the size of message 'i' for session 's' */
/* args are type 'session*'(s) and 'int'(i) */
/* i must not be 0 */

#define popmsguid(s,i) ((s)->uidl[(i)])
/* gives the 'char*' uid (unique signature) of 'int' message id */
